from .api import search, download, Video, get_random, parse_hanime_url, roll_search
import argparse
import sys
import time

SORT_OPTS_MAP = {
    "upload": "created_at_unix",
    "u": "created_at_unix",
    "views": "views",
    "v": "views",
    "likes": "likes",
    "l": "likes",
    "release": "released_at_unix",
    "r": "released_at_unix",
    "title": "title_sortable",
    "t": "title_sortable"
}

SORT_ORDER_MAP = {
    "a": "asc",
    "ascending": "asc",
    "d": "desc",
    "descending": "desc"
}

def verbose_download(video, res=1080, verbose=False):
    print(f"Downloading {video.title}...")
    download(video, res, verbose)

def output(video, res=1080, url=False, metadata=False, verbose=False):
    if url or metadata:
        if url:
            sources = video.at_resolution(res)

            print(f"{video.title}:")
            for i, j in sources.items():
                server, res = tuple(i.split("-"))
                print(f"{server}, {res}p: {j}")
            
            print()
        
        if metadata:
            tags_str = ", ".join(video.metadata.tags)
            print(f"URL: https://hanime.tv/videos/hentai/{video.slug}")
            print(f"Brand: {video.metadata.brand}")
            print(f"Likes: {video.metadata.likes}")
            print(f"Dislikes: {video.metadata.dislikes}")
            print(f"Views: {video.metadata.views}")
            print(f"Downloads: {video.metadata.downloads}")
            print(f"Tags: {tags_str}")
            print(f"Description:\n{video.metadata.description}\n")
    else:
        verbose_download(video, res, verbose)

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("video", nargs="*", help="Video URL or search term")
    parser.add_argument("--tags", "-t", help="Tags to search for", action="store", nargs="+", default=[])
    parser.add_argument("--broad-tag-match", help="Match videos including any tags specified by --tags", action="store_const", const="OR", default="AND")
    parser.add_argument("--blacklist", "-b", help="Blacklisted tags", action="store", nargs="+", default=[])
    parser.add_argument("--company", "-c", help="Companies/brands to filter by", action="store", nargs="+", default=[])
    parser.add_argument("--page", "-p", help="Page # of search results", default=1, type=int)
    parser.add_argument("--sort-by", "-s", help="Sorting method for search results ([u]pload, [v]iews, [l]ikes, [r]elease, [t]itle)", default="title")
    parser.add_argument("--sort-order", "-w", help="Order of sorting ([a]scending or [d]escending)", default="ascending")
    parser.add_argument("--roll-search", "-R", help="Roll all search pages into one long page, useful for large-volume downloads", action="store_true", default=False)
    parser.add_argument("--resolution", "-r", help="Resolution of download, default 1080", default=1080, type=int)
    parser.add_argument("--index", "-i", help="Index of search results to download", action="store", nargs="+", type=int, default=[])
    parser.add_argument("--all", "-a", help="Download all search results in page", action="store_true", default=False)
    parser.add_argument("--url", "-u", help="Show urls of the source video, do not download", action="store_true", default=False)
    parser.add_argument("--metadata", "-m", help="Show metadata of the source video, do not download", action="store_true", default=False)
    parser.add_argument("--verbose", "-v", help="Enable verbose logging for video download", action="store_true", default=False)
    args = parser.parse_args()

    slugs = list(map(parse_hanime_url, args.video))

    if len(sys.argv) == 1:
        parser.print_help()
        exit(1)

    if None not in slugs:
        for slug in slugs:
            video = Video.from_slug(slug)

            output(video, args.resolution, args.url, args.metadata)
    else:
        query = " ".join(args.video)
        
        if query == "ALL":
            query = ""

        if query == "random":
            seed = int(time.time()*1000)
            results = get_random(seed)
            
            print("Random:")
            for result in results:
                print(f"{result.title}")
            
            exit(0)
        elif query == "new-uploads":
            num_pages, results = search("", order_by="created_at_unix", ordering="desc")
        elif query == "new-releases":
            num_pages, results = search("", order_by="released_at_unix", ordering="desc")
        else:
            sort_by = args.sort_by
            sort_order = args.sort_order
            
            if sort_by not in SORT_OPTS_MAP:
                print(f'Unknown sort method "{args.sort_by}", using sort by title')
                sort_by = "title"
            if sort_order not in SORT_ORDER_MAP:
                print(f'Unknown sort order "{args.sort_order}", using ascending order')
                sort_order = "ascending"
            
            search_kwargs = {
                "blacklist": args.blacklist,
                "brands": args.company,
                "tags": args.tags,
                "page": args.page - 1,
                "tags_mode": args.broad_tag_match,
                "order_by": SORT_OPTS_MAP[sort_by],
                "ordering": SORT_ORDER_MAP[sort_order]
            }
            
            if args.roll_search:
                num_pages, results = 1, roll_search(query, **search_kwargs)
            else:
                num_pages, results = search(query, **search_kwargs)
        
        if len(results) > 1 and args.index == [] and not args.all:
            print(f'Found more than one match for "{query}"')
            print(f"Page {args.page} of {num_pages}")
            for index, result in enumerate(results):
                print(f"{index + 1}\t{result.title}")
            
            print("\nSpecify results to download with --index/-i, or download all results shown with --all/-a")
        else:
            if args.index and not args.all:
                for i in args.index:
                    if i <= len(results):
                        output(results[i-1].video, args.resolution, args.url, args.metadata, args.verbose)
            elif args.all or len(results) == 1:
                for result in results:
                    output(result.video, args.resolution, args.url, args.metadata, args.verbose)
            elif len(results) == 0:
                print(f'No results for "{query}"')

if __name__ == "__main__":
    main()



import requests
import json
import re
import subprocess
from urllib.parse import urlparse
from youtube_dl import YoutubeDL

class SearchResult:
    def __init__(self, slug, title):
        self.title = title
        self.slug = slug
    
    @property
    def video(self):
        return Video.from_slug(self.slug)
    
    def __str__(self):
        return f"<Result {self.slug}: {self.title}>"
    
    __repr__ = __str__

class Video:
    def __init__(self, json_enc):
        self.title = json_enc["hentai_video"]["name"]
        self.slug = json_enc["hentai_video"]["slug"]
        self.sources = {}
        metadata = {}
        
        for server in json_enc["videos_manifest"]["servers"]:
            for source in server["streams"]:
                name = server["name"]
                res = source["height"]
                vid = source["id"]
                self.sources[f"{name}-{res}"] = f"https://weeb.hanime.tv/weeb-api-cache/api/v8/m3u8s/{vid}.m3u8"
        
        metadata["brand"] = json_enc["hentai_video"]["brand"]
        metadata["likes"] = json_enc["hentai_video"]["likes"]
        metadata["dislikes"] = json_enc["hentai_video"]["dislikes"]
        metadata["views"] = json_enc["hentai_video"]["views"]
        metadata["tags"] = list(map(lambda i: i["text"], json_enc["hentai_video"]["hentai_tags"]))
        metadata["thumbnail"] = "https://i1.wp.com/static-assets.droidbuzz.top" + urlparse(json_enc["hentai_video"]["poster_url"]).path
        metadata["cover"] = "https://i1.wp.com/static-assets.droidbuzz.top" + urlparse(json_enc["hentai_video"]["cover_url"]).path
        metadata["downloads"] = json_enc["hentai_video"]["downloads"]
        metadata["monthly_rank"] = json_enc["hentai_video"]["monthly_rank"]
        metadata["description"] = re.compile(r'<[^>]+>').sub("", json_enc["hentai_video"]["description"])
        self.metadata = type("Metadata", (), metadata)()
    
    @staticmethod
    def from_slug(slug):
        r = requests.get(
            f"https://hw.hanime.tv/api/v8/video?id={slug}",
            headers = {
                "User-Agent": "Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.106 Safari/537.36"
            }
        )
        json_enc = r.json()
        return Video(json_enc)

    def at_resolution(self, res):
        max_res = int(max(self.sources, key=lambda source: int(source.split("-")[1])).split("-")[1])
        
        if max_res <= res:
            res = max_res

        sources = {
            x: self.sources[x] for x in self.sources if x.endswith(str(res))
        }
        
        return sources
    
    def __str__(self):
        return f'<Video {self.slug}: "{self.title}">'
    
    __repr__ = __str__

def parse_hanime_url(url):
    if "hanime.tv" in url:
        return url.split("/hentai/")[1]
    else:
        return None

def download(video, res=1080, verbose=False):
    true_res = list(video.at_resolution(res).keys())[0].split("-")[1]
    source = list(video.at_resolution(res).values())[0]
    opts = {
        "outtmpl": f"{video.slug}-{true_res}p.mp4"
    }
    
    if not verbose:
        print("asdasad")
        opts["external_downloader_args"] = ["-loglevel","warning","-stats"]
    
    with YoutubeDL(opts) as dl:
        dl.download([source])

def get_random(seed):
    j = requests.get("https://members.hanime.tv/rapi/v7/hentai_videos",
        params = {
            "source": "randomize",
            "r": str(seed)
        }
    ).json()
    results = []

    for result in j["hentai_videos"]:
        results.append(SearchResult(result["slug"], result["name"]))

    return results

def search(query, blacklist=[], brands=[], order_by="title_sortable", ordering="asc", page=0, tags=[], tags_mode="AND"):
    """
    ```
    Args:

        query: Text to search for

        blacklist: List of blacklisted tags

        brands: List of brands to filter by

        order_by:

            "created_at_unix": Order by creation date

            "views": Order by number of views

            "likes": Order by number of likes

            "released_at_unix": Order by release date

            "title_sortable": Order by alphabetic order of titles
        
        ordering:

            "desc": Descending order

            "asc": Ascending order

        page: Page number

        tags: List of tags to filter by

        tags_mode:

            "AND": Search for videos that have all tags listed in the tags parameter

            "OR": Search for videos that have any tags listed in the tags parameter
    ```
    """

    results = []

    r = requests.post("https://search.htv-services.com/",
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.106 Safari/537.36",
            "Content-Type": "application/json;charset=UTF-8"
        },
        json={
            "blacklist": blacklist,
            "brands": brands,
            "order_by": order_by,
            "ordering": ordering,
            "page": page,
            "search_text": query,
            "tags": tags,
            "tags_mode": tags_mode,
        }
    ).json()
    
    j = json.loads(r["hits"])

    for result in j:
        results.append(SearchResult(result["slug"], result["name"]))
    
    return r["nbPages"], results

def roll_search(query, blacklist=[], brands=[], order_by="title_sortable", ordering="asc", page=0, tags=[], tags_mode="AND"):
    num_pages, results = search(query, blacklist=blacklist, brands=brands, order_by=order_by, ordering=ordering, tags=tags, tags_mode=tags_mode)
    
    for p in range(num_pages):
        results += search(query, blacklist=blacklist, brands=brands, order_by=order_by, ordering=ordering, page=p, tags=tags, tags_mode=tags_mode)[1]
    
    return results



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hanimetv']

package_data = \
{'': ['*']}

install_requires = \
['requests', 'youtube_dl']

entry_points = \
{'console_scripts': ['htv = hanimetv.cli:main']}

setup_kwargs = {
    'name': 'hanimetv',
    'version': '1.1.4',
    'description': 'CLI downloader tool for hanime.tv',
    'long_description': '# hanimetv\nCLI tool for downloading hentai from hanime.tv\n## Installation\nInstall `ffmpeg` with whatever package manager you use, then run `pip install hanimetv`.\n## CLI Usage\n```\nusage: htv [-h] [--tags TAGS [TAGS ...]] [--broad-tag-match] [--blacklist BLACKLIST [BLACKLIST ...]] [--company COMPANY [COMPANY ...]] [--page PAGE] [--sort-by SORT_BY] [--sort-order SORT_ORDER] [--roll-search] [--resolution RESOLUTION] [--index INDEX [INDEX ...]] [--all] [--url] [--metadata] [video [video ...]]\n\npositional arguments:\n  video                 Video URL or search term\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --tags TAGS [TAGS ...], -t TAGS [TAGS ...]\n                        Tags to search for\n  --broad-tag-match     Match videos including any tags specified by --tags\n  --blacklist BLACKLIST [BLACKLIST ...], -b BLACKLIST [BLACKLIST ...]\n                        Blacklisted tags\n  --company COMPANY [COMPANY ...], -c COMPANY [COMPANY ...]\n                        Companies/brands to filter by\n  --page PAGE, -p PAGE  Page # of search results\n  --sort-by SORT_BY, -s SORT_BY\n                        Sorting method for search results ([u]pload, [v]iews, [l]ikes, [r]elease, [t]itle)\n  --sort-order SORT_ORDER, -w SORT_ORDER\n                        Order of sorting ([a]scending or [d]escending)\n  --roll-search, -R     Roll all search pages into one long page, useful for large-volume downloads\n  --resolution RESOLUTION, -r RESOLUTION\n                        Resolution of download, default 1080\n  --index INDEX [INDEX ...], -i INDEX [INDEX ...]\n                        Index of search results to download\n  --all, -a             Download all search results in page\n  --url, -u             Show urls of the source video, do not download\n  --metadata, -m        Show metadata of the source video, do not download\n```\nThere are some special search terms you can use.\n - `htv ALL` - Shows all results matching filters\n - `htv random` - Random list of hentai\n - `htv new-uploads` - Shows the newest uploads\n - `htv new-releases` - Shows the newest releases\n## FAQ\n - Can this download 1080p videos without Premium?\n\nYes. It queries the backend directly to get 1080p videos without needing an account.\n - How do I download all videos matching a filter?\n\n`htv ALL -R -a <FILTER>` will do this.\nSome examples:\n\nDownload all videos from a brand:\n\n`htv ALL -R -a -c "<BRAND>"`\n\nDownload all videos matching a particular tag: \n\n`htv ALL -R -a -t "<TAG>"`\n - When I search for brand or tag XYZ, it shows empty search results\n \n If you are using tag, company, or blacklist filtering for search, you will need to make sure that the filters have quotes around them and are spelled and capitalized correctly.\n \n Example: `htv ALL -c majin label` will show empty search results, but `htv ALL -c "Magin Label"` will show the correct results.\n- How can I send you death threats or feature requests?\n\nSend an email to rxqv@waifu.club and I probably won\'t read it unless you ask nicely.',
    'author': 'rxqv',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/rxqv/htv',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
